// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "AirbridgeInAppPurchaseEnvironment.h"
#include "AirbridgeLogLevel.h"
#include "AirbridgeSettings.generated.h"

UCLASS(config = Engine)
class UAirbridgeSettings : public UObject
{
	GENERATED_BODY()

public:
	UAirbridgeSettings(const FObjectInitializer& ObjectInitializer);

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "App Name"))
	FString AppName;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "App Token"))
	FString AppToken;
	
	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Log Level"))
	EAirbridgeLogLevel LogLevel;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "iOS URI Scheme"))
	FString IOSURIScheme;
	
	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Android URI Scheme"))
	FString AndroidURIScheme;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Custom Domain"))
	TArray<FString> CustomDomain;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Session Timeout Seconds"))
	int64 SessionTimeoutSeconds;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "User Info Hash Enabled"))
	bool UserInfoHashEnabled;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Location Collection Enabled"))
	bool LocationCollectionEnabled;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Track Airbridge Link Only"))
	bool TrackAirbridgeLinkOnly;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Auto Start Tracking Enabled"))
	bool AutoStartTrackingEnabled;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Facebook Deferred App Link Enabled"))
	bool FacebookDeferredAppLinkEnabled;
	
	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "iOS Tracking Authorize Timeout Seconds"))
	int32 IOSTrackingAuthorizeTimeoutSeconds;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "SDK Signature Secret ID"))
	FString SdkSignatureSecretID;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "SDK Signature Secret"))
	FString SdkSignatureSecret;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Track In Session Life Cycle Event Enabled"))
	bool TrackInSessionLifeCycleEventEnabled;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Pause Event Transmit On Background Enabled"))
	bool PauseEventTransmitOnBackgroundEnabled;
	
	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Clear Event Buffer On Initialize Enabled"))
	bool ResetEventBufferEnabled;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "SDK Enabled"))
	bool SdkEnabled;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "App Market Identifier"))
	FString AppMarketIdentifier;
	
	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Event Buffer Count Limit"))
	int32 EventMaximumBufferCount;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Event Buffer Size Limit in Gibibyte"))
	double EventMaximumBufferSize;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Event Transmit Interval Seconds"))
	int64 EventTransmitIntervalSeconds;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Meta Install Referrer (Facebook App ID)"))
	FString FacebookAppId;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Is Handle Airbridge Deeplink Only"))
	bool IsHandleAirbridgeDeeplinkOnly;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "In-App Purchase Environment"))
	EAirbridgeInAppPurchaseEnvironment InAppPurchaseEnvironment;

	UPROPERTY(Config, EditAnywhere, Category = "Airbridge Configure", meta = (DisplayName = "Collect TCF Data Enabled"))
	bool CollectTCFDataEnabled;
};