// Copyright © 2023 ab180. All rights reserved.

#include "AirbridgeAndroidCallback.h"

#if PLATFORM_ANDROID
#include "AirbridgeAndroidCore.h"
#include "AirbridgeUnrealModule.h"
#include "Platforms/Android/Util/AirbridgeJNI.h"

#define AIRBRIDGE_UNREAL_MODULE_POINTER		(FAirbridgeUnrealModule::Get())
#define ANDROID_CORE_POINTER				(static_cast<AirbridgeAndroidCore*>((AIRBRIDGE_UNREAL_MODULE_POINTER)->GetAirbridge().Get()))

jint GetCallbackType(JNIEnv *Env, jobject This)
{
	if (This == nullptr) {
		return -1;
	}
	jclass AirbridgeCallbackClass = Env->GetObjectClass(This);

	jmethodID GetCallbackTypeMethod = Env->GetMethodID(AirbridgeCallbackClass, "getCallbackType", "()I");
	if (GetCallbackTypeMethod == nullptr) {
		return -1;
	}

	jint Type = Env->CallIntMethod(This, GetCallbackTypeMethod);
	return Type;
}

FString GetCallbackUUID(JNIEnv *Env, jobject This)
{
	if (This == nullptr) {
		return "";
	}
	jclass AirbridgeCallbackClass = Env->GetObjectClass(This);

	jmethodID GetUUIDMethod = Env->GetMethodID(AirbridgeCallbackClass, "getUUID", "()Ljava/lang/String;");
	if (GetUUIDMethod == nullptr) {
		return "";
	}

	FString UUID;
	return AirbridgeJNI::ConvertString(
			Env,
			(jstring) Env->CallObjectMethod(This, GetUUIDMethod),
			UUID
	) ? UUID : "";
}

extern "C" {
	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnSuccessCallback
	 * Method:    invoke
	 * Signature: (Ljava/lang/Object;)V
	 */
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnSuccessCallback_invoke
	(JNIEnv *Env, jobject This, jobject Result)
	{
		FString StringParam;
		FAirbridgeTrackingLink AirbridgeTrackingLinkParam;
		
		// Get Method UUID
		FString UUID = GetCallbackUUID(Env, This);
		
		switch (GetCallbackType(Env, This)) {
		case HandleDeeplink:
			if (Result == nullptr) { return; }	// Nullable
			if (AirbridgeJNI::GetUriString(Env, Result, StringParam))
			{
				// Check if module is loaded
				if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				{
					ANDROID_CORE_POINTER->ProcessDeeplink(StringParam);
				}
				// If module is not loaded
				else
				{
					AirbridgeAndroidCore::DeeplinkData = StringParam;
				}
			}
			break;
		case HandleDeferredDeeplink:
			if (Result == nullptr) { return; }	// Nullable
			if (AIRBRIDGE_UNREAL_MODULE_POINTER && AirbridgeJNI::GetUriString(Env, Result, StringParam))
			{
				ANDROID_CORE_POINTER->ProcessDeeplink(StringParam);	
			}
			break;
		case Click:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessVoidParamFunction(UUID);
			break;
		case Impression:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessVoidParamFunction(UUID);
			break;
		case FetchDeviceUUID:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER && AirbridgeJNI::ConvertString(Env, (jstring)Result, StringParam))
				ANDROID_CORE_POINTER->ProcessStringParamFunction(UUID, StringParam);
			break;
		case FetchAirbridgeGeneratedUUID:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER && AirbridgeJNI::ConvertString(Env, (jstring)Result, StringParam))
				ANDROID_CORE_POINTER->ProcessStringParamFunction(UUID, StringParam);
			break;
		case CreateTrackingLink:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER && AirbridgeJNI::ConvertAirbridgeTrackingLink(Env, Result, AirbridgeTrackingLinkParam))
				ANDROID_CORE_POINTER->ProcessAirbridgeTrackingLinkParamFunction(UUID, AirbridgeTrackingLinkParam);
			break;
		default:
			break;
		}
	}

	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnFailureCallback
	 * Method:    invoke
	 * Signature: (Ljava/lang/Throwable;)V
	 */
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnFailureCallback_invoke
	(JNIEnv *Env, jobject This, jthrowable Result)
	{
		FString ErrorMessage;
		if (!AirbridgeJNI::GetErrorMessage(Env, Result, ErrorMessage)) return;
		
		// Get Method UUID
		FString UUID = GetCallbackUUID(Env, This);
		
		switch (GetCallbackType(Env, This)) {
		case HandleDeeplink:
		case HandleDeferredDeeplink:
			break;
		case Click:
		case Impression:
		case FetchDeviceUUID:
		case FetchAirbridgeGeneratedUUID:
		case CreateTrackingLink:
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
				ANDROID_CORE_POINTER->ProcessErrorMessageParamFunction(UUID, ErrorMessage);
			break;
		default:
			break;
		}
	}

	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnAttributionResultReceiveListener
	 * Method:    onAttributionResultReceived
	 * Signature: (Ljava/util/Map;)V
	 */
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnAttributionResultReceiveListener_onAttributionResultReceived
	(JNIEnv *Env, jobject This, jobject Map)
	{
		TMap<FString, FString> Data;
		if (AirbridgeJNI::ConvertMap(Env, Map, Data))
		{
			// Check if module is loaded
			if (AIRBRIDGE_UNREAL_MODULE_POINTER)
			{
				ANDROID_CORE_POINTER->ProcessOnAttributionResultReceived(Data);
			}
			// If module is not loaded
			else
			{
				AirbridgeAndroidCore::AttributionData = Data;
			}
		}
	}

	/*
	 * Class:     co_ab180_airbridge_ue_callback_AirbridgeOnInAppPurchaseReceiveListener
	 * Method:    onInAppPurchaseReceived
	 * Signature: (Lco/ab180/airbridge/AirbridgeInAppPurchase;)V
	*/
	JNIEXPORT void JNICALL Java_co_ab180_airbridge_ue_callback_AirbridgeOnInAppPurchaseReceiveListener_onInAppPurchaseReceived
	(JNIEnv *Env, jobject This, jobject InAppPurchase)
	{
		if (InAppPurchase == nullptr) return;

		// Check if module is loaded
		if (AIRBRIDGE_UNREAL_MODULE_POINTER)
		{
			ANDROID_CORE_POINTER->ProcessOnInAppPurchaseReceived(UAirbridgeInAppPurchase::CreateObject(InAppPurchase));
		}
	}
}
#endif
