// Copyright © 2023 ab180. All rights reserved.

#include "AirbridgeAndroidCore.h"
#include "Blueprints/AirbridgeCallbacks.h"
#include "UObject/UObjectIterator.h"
#include "Misc/Guid.h"

#if PLATFORM_ANDROID
#include "Android/AndroidJNI.h"
#include "Android/AndroidApplication.h"

#include "Platforms/Android/Util/AirbridgeJNI.h"
#endif

FString AirbridgeAndroidCore::DeeplinkData;
TMap<FString, FString> AirbridgeAndroidCore::AttributionData;

AirbridgeAndroidCore::AirbridgeAndroidCore()
{
}

AirbridgeAndroidCore::~AirbridgeAndroidCore()
{
}

#pragma region Core

void AirbridgeAndroidCore::EnableSDK()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeEnableSDK");
#endif
}

void AirbridgeAndroidCore::DisableSDK()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeDisableSDK");
#endif
}

bool AirbridgeAndroidCore::IsSDKEnabled()
{
#if PLATFORM_ANDROID
	return AirbridgeJNI::GameActivityCallMethod_Return_Boolean("airbridgeIsSDKEnabled");
#endif
	return false;
}

#pragma endregion

#pragma region Privacy

void AirbridgeAndroidCore::StartTracking()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeStartTracking");
#endif
}

void AirbridgeAndroidCore::StopTracking()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeStopTracking");
#endif
}

bool AirbridgeAndroidCore::IsTrackingEnabled()
{
#if PLATFORM_ANDROID
	return AirbridgeJNI::GameActivityCallMethod_Return_Boolean("airbridgeIsTrackingEnabled");
#endif
	return false;
}

#pragma endregion

#pragma region Deeplink

void AirbridgeAndroidCore::SetOnDeeplinkReceived(const AirbridgeStringParamFunction OnDeeplinkReceived)
{
	// Only the first callback is valid
	if (HandleDeeplinkListener) return;
	
	HandleDeeplinkListener = OnDeeplinkReceived;
	
	if (DeeplinkData.IsEmpty())
	{
#if PLATFORM_ANDROID
		AirbridgeJNI::GameActivityCallMethod("airbridgeHandleDeferredDeeplink");
#endif
	}
	else
	{
		HandleDeeplink();
	}
}

#pragma endregion

#pragma region Data Collection

void AirbridgeAndroidCore::SetUserID(const FString& Id)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeSetUserID", Id);
#endif
}

void AirbridgeAndroidCore::ClearUserID()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearUserID");
#endif
}

void AirbridgeAndroidCore::SetUserEmail(const FString& Email)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeSetUserEmail", Email);
#endif
}

void AirbridgeAndroidCore::ClearUserEmail()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearUserEmail");
#endif
}

void AirbridgeAndroidCore::SetUserPhone(const FString& Phone)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeSetUserPhone", Phone);
#endif
}

void AirbridgeAndroidCore::ClearUserPhone()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearUserPhone");
#endif
}

void AirbridgeAndroidCore::SetUserAttribute(const FString& Key, const int32 Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetUserAttribute", "(Ljava/lang/String;I)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, Value);
    }
#endif
}

void AirbridgeAndroidCore::SetUserAttribute(const FString& Key, const int64 Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetUserAttribute", "(Ljava/lang/String;J)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, Value);
    }
#endif
}

void AirbridgeAndroidCore::SetUserAttribute(const FString& Key, const float Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetUserAttribute", "(Ljava/lang/String;F)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, Value);
    }
#endif
}

void AirbridgeAndroidCore::SetUserAttribute(const FString& Key, const double Value)
{
#if PLATFORM_ANDROID
	if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
	{
		jmethodID Method = FJavaWrapper::FindMethod(Env,
			FJavaWrapper::GameActivityClassID,
			"airbridgeSetUserAttribute", "(Ljava/lang/String;D)V", false);
		jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
		FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, Value);
	}
#endif
}

void AirbridgeAndroidCore::SetUserAttribute(const FString& Key, const bool Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetUserAttribute", "(Ljava/lang/String;Z)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, Value);
    }
#endif
}

void AirbridgeAndroidCore::SetUserAttribute(const FString& Key, const FString& Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetUserAttribute", "(Ljava/lang/String;Ljava/lang/String;)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        jstring jValue = AirbridgeJNI::GetJavaString(Env, Value);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, jValue);
    }
#endif
}

void AirbridgeAndroidCore::RemoveUserAttribute(const FString& Key)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeRemoveUserAttribute", Key);
#endif
}

void AirbridgeAndroidCore::ClearUserAttributes()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearUserAttributes");
#endif
}

void AirbridgeAndroidCore::SetUserAlias(const FString& Key, const FString& Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetUserAlias", "(Ljava/lang/String;Ljava/lang/String;)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        jstring jValue = AirbridgeJNI::GetJavaString(Env, Value);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, jValue);
    }
#endif
}

void AirbridgeAndroidCore::RemoveUserAlias(const FString& Key)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeRemoveUserAlias", Key);
#endif
}

void AirbridgeAndroidCore::ClearUserAlias()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearUserAlias");
#endif
}

void AirbridgeAndroidCore::ClearUser()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearUser");
#endif
}

void AirbridgeAndroidCore::SetDeviceAlias(const FString& Key, const FString& Value)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
            FJavaWrapper::GameActivityClassID,
            "airbridgeSetDeviceAlias", "(Ljava/lang/String;Ljava/lang/String;)V", false);
        jstring jKey = AirbridgeJNI::GetJavaString(Env, Key);
        jstring jValue = AirbridgeJNI::GetJavaString(Env, Value);
        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method, jKey, jValue);
    }
#endif
}

void AirbridgeAndroidCore::RemoveDeviceAlias(const FString& Key)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeRemoveDeviceAlias", Key);
#endif
}

void AirbridgeAndroidCore::ClearDeviceAlias()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeClearDeviceAlias");
#endif
}

void AirbridgeAndroidCore::RegisterPushToken(const FString& Token)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeRegisterPushToken", Token);
#endif
}

#pragma endregion

#pragma region Event

void AirbridgeAndroidCore::TrackEvent(
	const FString& Category,
	const UAirbridgeMap* SemanticAttributes,
	const UAirbridgeMap* CustomAttributes
)
{
#if PLATFORM_ANDROID
    if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
    {
        jmethodID Method = FJavaWrapper::FindMethod(Env,
           FJavaWrapper::GameActivityClassID,
           "airbridgeTrackEvent",
           "(Ljava/lang/String;Ljava/util/HashMap;Ljava/util/HashMap;)V", false);
    	
    	jstring jCategory = AirbridgeJNI::GetJavaString(Env, Category);
    	jobject jSemanticAttributes = AirbridgeJNI::GetJavaHashMap(Env, SemanticAttributes);
    	jobject jCustomAttributes = AirbridgeJNI::GetJavaHashMap(Env, CustomAttributes);

        FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method,
        	jCategory, jSemanticAttributes, jCustomAttributes);
    }
#endif
}

#pragma endregion

#pragma region Placement

void AirbridgeAndroidCore::Click(
	const FString& TrackingLink,
	const AirbridgeVoidParamFunction OnSuccess,
	const AirbridgeStringParamFunction OnFailure
)
{
	FString UUID = FGuid::NewGuid().ToString();
	SetVoidParamFunctionMap(UUID, OnSuccess);
	SetErrorMessageParamFunctionMap(UUID, OnFailure);
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_TwoParam_String_String("airbridgeClick", TrackingLink, UUID);
#endif
}

void AirbridgeAndroidCore::Impression(
	const FString& TrackingLink,
	const AirbridgeVoidParamFunction OnSuccess,
	const AirbridgeStringParamFunction OnFailure
)
{
	FString UUID = FGuid::NewGuid().ToString();
	SetVoidParamFunctionMap(UUID, OnSuccess);
	SetErrorMessageParamFunctionMap(UUID, OnFailure);
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_TwoParam_String_String("airbridgeImpression", TrackingLink, UUID);
#endif
}

#pragma endregion

#pragma region Publication

void AirbridgeAndroidCore::FetchDeviceUUID(const AirbridgeStringParamFunction OnSuccess, const AirbridgeStringParamFunction OnFailure)
{
	FString UUID = FGuid::NewGuid().ToString();
	SetStringParamFunctionMap(UUID, OnSuccess);
	SetErrorMessageParamFunctionMap(UUID, OnFailure);
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeFetchDeviceUUID", UUID);
#endif
}

void AirbridgeAndroidCore::FetchAirbridgeGeneratedUUID(const AirbridgeStringParamFunction OnSuccess, const AirbridgeStringParamFunction OnFailure)
{
	FString UUID = FGuid::NewGuid().ToString();
	SetStringParamFunctionMap(UUID, OnSuccess);
	SetErrorMessageParamFunctionMap(UUID, OnFailure);
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeFetchAirbridgeGeneratedUUID", UUID);
#endif
}

void AirbridgeAndroidCore::SetOnAttributionReceived(const AirbridgeMapParamFunction OnAttributionReceived)
{
	// Only the first callback is valid
	if (AirbridgeOnAttributionResultReceiveListener) return;
	
	AirbridgeOnAttributionResultReceiveListener = OnAttributionReceived;	
	
	HandleOnAttributionReceived();
}

void AirbridgeAndroidCore::CreateTrackingLink(
	const FString& Channel,
	const UAirbridgeMap* Option,
	const AirbridgeTrackingLinkParamFunction OnSuccess,
	const AirbridgeStringParamFunction OnFailure
)
{
	FString UUID = FGuid::NewGuid().ToString();
	SetAirbridgeTrackingLinkParamFunctionMap(UUID, OnSuccess);
	SetErrorMessageParamFunctionMap(UUID, OnFailure);
#if PLATFORM_ANDROID
	if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
	{
		jmethodID Method = FJavaWrapper::FindMethod(Env,
		   FJavaWrapper::GameActivityClassID,
		   "airbridgeCreateTrackingLink",
		   "(Ljava/lang/String;Ljava/util/HashMap;Ljava/lang/String;)V", false);
		
		jstring jChannel = AirbridgeJNI::GetJavaString(Env, Channel);
		jobject jOption = AirbridgeJNI::GetJavaHashMap(Env, Option);
		jstring jUUID = AirbridgeJNI::GetJavaString(Env, UUID);
		
		FJavaWrapper::CallVoidMethod(Env, FJavaWrapper::GameActivityThis, Method,
			jChannel, jOption, jUUID);
	}
#endif
}

#pragma endregion

#pragma region Hybrid

FString AirbridgeAndroidCore::CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript)
{
#if PLATFORM_ANDROID
	if (JNIEnv* Env = FAndroidApplication::GetJavaEnv())
	{
		jmethodID Method = FJavaWrapper::FindMethod(Env,
			FJavaWrapper::GameActivityClassID,
			"airbridgeCreateWebInterfaceScript",
			"(Ljava/lang/String;Ljava/lang/String;)Ljava/lang/String;", false);

		jstring jWebToken = AirbridgeJNI::GetJavaString(Env, WebToken);
		jstring jPostMessageScript = AirbridgeJNI::GetJavaString(Env, PostMessageScript);
		jstring jWebInterfaceScript = (jstring) FJavaWrapper::CallObjectMethod(Env, FJavaWrapper::GameActivityThis, Method, jWebToken, jPostMessageScript);
		
		// Copies the new key into its final C string buffer
		const char* cWebInterfaceScript = Env->GetStringUTFChars(jWebInterfaceScript, nullptr);
		if (cWebInterfaceScript == nullptr) {
			return "";  /* OutOfMemoryError already thrown */
		}
	
		FString toReturn = FString(cWebInterfaceScript);

		//release heap memory
		Env->ReleaseStringUTFChars(jWebInterfaceScript, cWebInterfaceScript);
		Env->DeleteLocalRef(jWebInterfaceScript);
		
		return toReturn;
	}
	return "";
#endif
	return "";
}

void AirbridgeAndroidCore::HandleWebInterfaceCommand(const FString& Command)
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod_OneParam_String("airbridgeHandleWebInterfaceCommand", Command);
#endif
}

#pragma endregion

#pragma region IAP

void AirbridgeAndroidCore::StartInAppPurchaseTracking()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeStartInAppPurchaseTracking");
#endif
}

void AirbridgeAndroidCore::StopInAppPurchaseTracking()
{
#if PLATFORM_ANDROID
	AirbridgeJNI::GameActivityCallMethod("airbridgeStopInAppPurchaseTracking");
#endif
}

bool AirbridgeAndroidCore::IsInAppPurchaseTrackingEnabled()
{
#if PLATFORM_ANDROID
	return AirbridgeJNI::GameActivityCallMethod_Return_Boolean("airbridgeIsInAppPurchaseTrackingEnabled");
#endif
	return false;
}

void AirbridgeAndroidCore::SetOnInAppPurchaseReceived(const AirbridgeInAppPurchaseParamFunction OnReceived)
{
	// Only the first callback is valid
	if (AirbridgeOnInAppPurchaseReceiveListener) return;
	
	AirbridgeOnInAppPurchaseReceiveListener = OnReceived;
}

#pragma endregion

void AirbridgeAndroidCore::SetErrorMessageParamFunctionMap(const FString& MethodUUID, const AirbridgeStringParamFunction& Function)
{
	if (!Function) return;
	ErrorMessageParamFunctionMap.Add(MethodUUID, Function);
}

void AirbridgeAndroidCore::SetStringParamFunctionMap(const FString& MethodUUID, const AirbridgeStringParamFunction& Function)
{
	if (!Function) return;
	StringParamFunctionMap.Add(MethodUUID, Function);
}

void AirbridgeAndroidCore::SetVoidParamFunctionMap(const FString& MethodUUID, const AirbridgeVoidParamFunction& Function)
{
	if (!Function) return;
	VoidParamFunctionMap.Add(MethodUUID, Function);
}

void AirbridgeAndroidCore::SetAirbridgeTrackingLinkParamFunctionMap(const FString& MethodUUID, const AirbridgeTrackingLinkParamFunction& Function)
{
	if (!Function) return;
	AirbridgeTrackingLinkParamFunctionMap.Add(MethodUUID, Function);
}

void AirbridgeAndroidCore::ReleaseCallbackMethod(const FString& MethodUUID)
{
	ErrorMessageParamFunctionMap.Remove(MethodUUID);
	StringParamFunctionMap.Remove(MethodUUID);
	VoidParamFunctionMap.Remove(MethodUUID);
	AirbridgeTrackingLinkParamFunctionMap.Remove(MethodUUID);
}

void AirbridgeAndroidCore::ProcessErrorMessageParamFunction(const FString& MethodUUID, const FString& Param)
{
	// Check if callback method exists
	if (ErrorMessageParamFunctionMap.Contains(MethodUUID))
	{
		// Call callback method
		AirbridgeStringParamFunction CallbackFunction = ErrorMessageParamFunctionMap[MethodUUID];
		if (CallbackFunction) CallbackFunction(Param);

		ReleaseCallbackMethod(MethodUUID);
	}
}

void AirbridgeAndroidCore::ProcessStringParamFunction(const FString& MethodUUID, const FString& Param)
{
	// Check if callback method exists
	if (StringParamFunctionMap.Contains(MethodUUID))
	{
		// Call callback method
		AirbridgeStringParamFunction CallbackFunction = StringParamFunctionMap[MethodUUID];
		if (CallbackFunction) CallbackFunction(Param);

		ReleaseCallbackMethod(MethodUUID);
	}
}

void AirbridgeAndroidCore::ProcessVoidParamFunction(const FString& MethodUUID)
{
	// Check if callback method exists
	if (VoidParamFunctionMap.Contains(MethodUUID))
	{
		// Call callback method
		AirbridgeVoidParamFunction CallbackFunction = VoidParamFunctionMap[MethodUUID];
		if (CallbackFunction) CallbackFunction();
	
		ReleaseCallbackMethod(MethodUUID);
	}
}

void AirbridgeAndroidCore::ProcessAirbridgeTrackingLinkParamFunction(const FString& MethodUUID, const FAirbridgeTrackingLink& Param)
{
	// Check if callback method exists
	if (AirbridgeTrackingLinkParamFunctionMap.Contains(MethodUUID))
	{
		// Call callback method
		AirbridgeTrackingLinkParamFunction CallbackFunction = AirbridgeTrackingLinkParamFunctionMap[MethodUUID];
		if (CallbackFunction) CallbackFunction(Param);

		ReleaseCallbackMethod(MethodUUID);
	}
}

#pragma region Deeplink

void AirbridgeAndroidCore::ProcessDeeplink(const FString& Uri)
{
	DeeplinkData = Uri;
	HandleDeeplink();
}

void AirbridgeAndroidCore::HandleDeeplink()
{
	if (DeeplinkData.IsEmpty()) return;

	// Use Blueprints
	bool UseBroadcast = false;
	for (TObjectIterator<UAirbridgeCallbacks> Itr; Itr; ++Itr)
	{
		Itr->OnDeeplinkReceived.Broadcast(DeeplinkData);
		UseBroadcast = true;
	}
	if (UseBroadcast)
	{
		DeeplinkData.Reset();
		return;
	}

	// Use C++
	if (HandleDeeplinkListener)
	{
		HandleDeeplinkListener(DeeplinkData);
		DeeplinkData.Reset();
	}
}

#pragma endregion

#pragma region Attribution

void AirbridgeAndroidCore::ProcessOnAttributionResultReceived(const TMap<FString, FString>& Data)
{
	AttributionData = Data;
	HandleOnAttributionReceived();
}

void AirbridgeAndroidCore::HandleOnAttributionReceived()
{
	if (AttributionData.Num() == 0) return;

	// Use Blueprints
	bool UseBroadcast = false;
	for (TObjectIterator<UAirbridgeCallbacks> Itr; Itr; ++Itr)
	{
		Itr->OnAttributionReceived.Broadcast(FAirbridgeBlueprintMap { AttributionData });
		UseBroadcast = true;
	}
	if (UseBroadcast)
	{
		AttributionData.Empty();
		return;
	}

	// Use C++
	if (AirbridgeOnAttributionResultReceiveListener)
	{
		AirbridgeOnAttributionResultReceiveListener(AttributionData);
		AttributionData.Empty();
	}
}

#pragma endregion

#pragma region IAP

void AirbridgeAndroidCore::ProcessOnInAppPurchaseReceived(UAirbridgeInAppPurchase* InAppPurchase)
{
	// Use Blueprints
	bool UseBroadcast = false;
	for (TObjectIterator<UAirbridgeCallbacks> Itr; Itr; ++Itr)
	{
		Itr->OnInAppPurchaseReceived.Broadcast(InAppPurchase);
		UseBroadcast = true;
	}
	if (UseBroadcast)
	{
		return;
	}
	
	// Use C++
	if (AirbridgeOnInAppPurchaseReceiveListener)
	{
		AirbridgeOnInAppPurchaseReceiveListener(InAppPurchase);
	}
}

#pragma endregion