// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "AirbridgeUnrealModule.h"
#include "Interfaces/AirbridgeCore.h"

#define AIRBRIDGE_UNREAL_MODULE_POINTER	(FAirbridgeUnrealModule::Get())
#define IOS_CORE_POINTER                (static_cast<AirbridgeIOSCore*>((AIRBRIDGE_UNREAL_MODULE_POINTER)->GetAirbridge().Get()))

/**
 * 
 */
class AirbridgeIOSCore : public IAirbridgeCore
{
public:
	AirbridgeIOSCore();
	virtual ~AirbridgeIOSCore();

#pragma region Core

	virtual void EnableSDK() override;
	virtual void DisableSDK() override;
	virtual bool IsSDKEnabled() override;

	// virtual void StartInAppPurchaseTracking() override;
	// virtual void StopInAppPurchaseTracking() override;
	// virtual bool IsInAppPurchaseTrackingEnabled() override;

#pragma endregion

#pragma region Privacy

	virtual void StartTracking() override;
	virtual void StopTracking() override;
	virtual bool IsTrackingEnabled() override;

#pragma endregion

#pragma region Deeplink

	virtual void SetOnDeeplinkReceived(const AirbridgeStringParamFunction OnDeeplinkReceived) override;

#pragma endregion

#pragma region Data Collection

	virtual void SetUserID(const FString& Id) override;
	virtual void ClearUserID() override;
	virtual void SetUserEmail(const FString& Email) override;
	virtual void ClearUserEmail() override;
	virtual void SetUserPhone(const FString& Phone) override;
	virtual void ClearUserPhone() override;
	virtual void SetUserAttribute(const FString& Key, const int32 Value) override;
	virtual void SetUserAttribute(const FString& Key, const int64 Value) override;
	virtual void SetUserAttribute(const FString& Key, const float Value) override;
	virtual void SetUserAttribute(const FString& Key, const double Value) override;
	virtual void SetUserAttribute(const FString& Key, const bool Value) override;
	virtual void SetUserAttribute(const FString& Key, const FString& Value) override;
	virtual void RemoveUserAttribute(const FString& Key) override;
	virtual void ClearUserAttributes() override;
	virtual void SetUserAlias(const FString& Key, const FString& Value) override;
	virtual void RemoveUserAlias(const FString& Key) override;
	virtual void ClearUserAlias() override;
	virtual void ClearUser() override;

	virtual void SetDeviceAlias(const FString& Key, const FString& Value) override;
	virtual void RemoveDeviceAlias(const FString& Key) override;
	virtual void ClearDeviceAlias() override;

	virtual void RegisterPushToken(const FString& Token) override;

#pragma endregion

#pragma region Event

	virtual void TrackEvent(
		const FString& Category,
		const UAirbridgeMap* SemanticAttributes = nullptr,
		const UAirbridgeMap* CustomAttributes = nullptr
	) override;

#pragma endregion

#pragma region Placement

	virtual void Click(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction OnSuccess = nullptr,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;
	virtual void Impression(
		const FString& TrackingLink,
		const AirbridgeVoidParamFunction OnSuccess = nullptr,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;

	virtual void CreateTrackingLink(
		const FString& Channel,
		const UAirbridgeMap* Option,
		const AirbridgeTrackingLinkParamFunction OnSuccess,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;

#pragma endregion

#pragma region Publication

	virtual void FetchDeviceUUID(
		const AirbridgeStringParamFunction OnSuccess,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;
	virtual void FetchAirbridgeGeneratedUUID(
		const AirbridgeStringParamFunction OnSuccess,
		const AirbridgeStringParamFunction OnFailure = nullptr
	) override;
	virtual void SetOnAttributionReceived(const AirbridgeMapParamFunction OnAttributionReceived) override;

#pragma endregion

#pragma region Hybrid

	virtual FString CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript) override;
	virtual void HandleWebInterfaceCommand(const FString& Command) override;

#pragma endregion

#pragma region IAP

	virtual void StartInAppPurchaseTracking() override;
	virtual void StopInAppPurchaseTracking() override;
	virtual bool IsInAppPurchaseTrackingEnabled() override;

	virtual void SetOnInAppPurchaseReceived(const AirbridgeInAppPurchaseParamFunction OnReceived) override;

#pragma endregion

#pragma region Deeplink

public:
    static FString DeeplinkData;
	void ProcessDeeplink(const FString& Uri);

private:
	AirbridgeStringParamFunction HandleDeeplinkListener = nullptr;
	void HandleDeeplink();

#pragma endregion 

#pragma region IAP
public:
    void ProcessOnInAppPurchaseReceived(id);

private:
    AirbridgeInAppPurchaseParamFunction AirbridgeOnInAppPurchaseReceiveListener = nullptr;
#pragma endregion

#pragma region Attribution
	
public:
	static TMap<FString, FString> AttributionData;
    void ProcessOnAttributionResultReceived(const TMap<FString, FString>& Data);

private:
	AirbridgeMapParamFunction AirbridgeOnAttributionResultReceiveListener = nullptr;
	void HandleOnAttributionReceived();

#pragma endregion
};
