#import "AUAppDelegate.h"
#import "AUAppSetting.h"

#include "Airbridge/Container/AirbridgeMap.h"

#import <StoreKit/StoreKit.h>

#include "AirbridgeIOSCore.h"
#include "AirbridgeUnrealModule.h"

@implementation AUAppDelegate

static AUAppDelegate *AUAppDelegateInstance = nil;

-(id)init
{
    if (self = [super init]) { }
    return (self);
}

+(void)load
{
    [super load];
    if (AUAppDelegateInstance == nil)
    {
        AUAppDelegateInstance = [[AUAppDelegate alloc] init];
        [AUAppDelegateInstance registerLifeCycleListener];
    }
}

+(AUAppDelegate *)getInstance
{
    return AUAppDelegateInstance;
}

-(void)registerLifeCycleListener
{
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(didFinishLaunching:)
                                                 name:UIApplicationDidFinishLaunchingNotification
                                               object:nil];
}

+(void)unregisterLifeCycleListener
{
    if (AUAppDelegateInstance != nil)
    {
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:UIApplicationDidFinishLaunchingNotification
                                                      object:nil];
    }
}

-(void)didFinishLaunching:(NSNotification *)notification
{
    if ([AUAppSetting.appName isEqualToString:@""] || [AUAppSetting.appToken isEqualToString:@""]) {
        return;
    }
    
    AirbridgeOptionBuilder *builder = [
        [AirbridgeOptionBuilder alloc]
        initWithName:AUAppSetting.appName token:AUAppSetting.appToken
    ];
    
    NSArray *customDomains = [AUAppSetting.customDomain componentsSeparatedByString:@" "];
    builder = [builder setSDKDevelopmentPlatform:@"unreal"];
    builder = [builder setAutoStartTrackingEnabled:AUAppSetting.autoStartTrackingEnabled];
    builder = [builder setAutoDetermineTrackingAuthorizationTimeoutWithSecond:AUAppSetting.trackingAuthorizeTimeoutSeconds];
    builder = [builder setSessionTimeoutWithSecond:AUAppSetting.sessionTimeoutSeconds];
    builder = [builder setHashUserInformationEnabled:AUAppSetting.userInfoHashEnabled];
    builder = [builder setTrackAirbridgeDeeplinkOnlyEnabled:AUAppSetting.trackAirbridgeLinkOnly];
    builder = [builder setLogLevel:(AirbridgeLogLevel)AUAppSetting.logLevel];
    if (customDomains.count != 0) {
        builder = [builder setTrackingLinkCustomDomains:customDomains];
    }
    
    if (([AUAppSetting.sdkSignatureSecretID isEqualToString:@""] || [AUAppSetting.sdkSignatureSecret isEqualToString:@""]) == NO) {
        builder = [builder setSDKSignatureWithId:AUAppSetting.sdkSignatureSecretID secret:AUAppSetting.sdkSignatureSecret];
    }
    
    builder = [builder setOnInAppPurchaseReceived:^(AirbridgeInAppPurchase * _Nonnull inAppPurchase) {
        if (!AIRBRIDGE_UNREAL_MODULE_POINTER) { return; }
        IOS_CORE_POINTER->ProcessOnInAppPurchaseReceived(inAppPurchase);
    }];
    builder = [builder setOnAttributionReceived:^(NSDictionary<NSString *,NSString *> * attribution) {
        TMap<FString, FString> Data;
        for (NSString *key in [attribution allKeys]) {
            Data.Emplace(key, attribution[key]);
        }
        
        if (AIRBRIDGE_UNREAL_MODULE_POINTER)
        {
            IOS_CORE_POINTER->ProcessOnAttributionResultReceived(Data);
        }
        
        else
        {
            AirbridgeIOSCore::AttributionData = Data;
        }
    }];
    
    builder = [builder setSDKAttributes: @{
        @"wrapperName": @"airbridge-unreal-sdk-restricted",
        @"wrapperVersion": @"4.4.0"
    }];
    
    builder = [builder setSDKWrapperOption: @{
        @"isHandleAirbridgeDeeplinkOnly": @(AUAppSetting.isHandleAirbridgeDeeplinkOnly)
    }];
    
    [Airbridge initializeSDKWithOption:[builder build]];
}

@end
