//
//  AirbridgeInAppPurchaseBridge.m
//  AirbridgeQA (IOS)
//
//  Created by mjgu on 4/7/25.
//  Copyright © 2025 Epic Games, Inc. All rights reserved.
//

#import "AirbridgeInAppPurchaseBridge.h"
#import <StoreKit/StoreKit.h>
#import <Airbridge/Airbridge.h>

@interface AirbridgeInAppPurchaseBridge ()

@property (nonatomic, strong) AirbridgeInAppPurchase *inAppPurchase;

@end

@implementation AirbridgeInAppPurchaseBridge

- (instancetype)initWithAirbridgeInAppPurchase:(AirbridgeInAppPurchase *)inAppPurchase {
    self = [super self];
    if (self) {
        self.inAppPurchase = inAppPurchase;
    }
    
    return self;
}

- (void)setSemanticAttributes:(NSDictionary *)semanticAttributes {
    [self.inAppPurchase setSemanticAttributes:semanticAttributes];
}

- (void)setCustomAttributes:(NSDictionary *)customAttributes {
    [self.inAppPurchase setCustomAttributes:customAttributes];
}

#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"

- (NSString *)fetchTransactionString {
    SKPaymentTransaction *transaction = self.inAppPurchase.transaction;
    NSDictionary *transactionDictionary = [self fetchTransaction:transaction];
    NSError *serializationError = nil;
    NSData *transactionData = [
        NSJSONSerialization
        dataWithJSONObject:transactionDictionary
        options:NSJSONWritingFragmentsAllowed
        error:&serializationError
    ];
    
    if (serializationError) { return @""; }
    
    NSString *transactionJSONString = [
        [NSString alloc]
        initWithData:transactionData
        encoding:NSUTF8StringEncoding
    ];
    
    if (!transactionJSONString) { return @""; }
    
    return transactionJSONString;
}

- (NSDictionary *)fetchTransaction:(SKPaymentTransaction *)transaction {
    NSMutableDictionary *dictionary = [NSMutableDictionary new];
    if (!transaction) { return dictionary; }
    
    if (transaction.payment) {
        [dictionary setValue:[self fetchPayment:transaction.payment] forKey:@"payment"];
    }
    if (transaction.error) {
        [dictionary setValue:transaction.error.localizedDescription forKey:@"error"];
    }
    if (transaction.originalTransaction) {
        [dictionary
         setValue:[self fetchTransaction:transaction.originalTransaction]
         forKey:@"originalTransaction"
        ];
    }
    if (transaction.transactionDate) {
        [dictionary setValue:@(transaction.transactionDate.timeIntervalSince1970) forKey:@"transactionDate"];
    }
    if (transaction.transactionIdentifier) {
        [dictionary setValue:transaction.transactionIdentifier forKey:@"transactionIdentifier"];
    }
    [dictionary setValue:@(transaction.transactionState) forKey:@"transactionState"];
    
    return dictionary;
}

- (NSDictionary *)fetchPayment:(SKPayment *)payment {
    NSMutableDictionary *dictionary = [NSMutableDictionary new];
    if (!payment) { return dictionary; }
    
    if (payment.paymentDiscount) {
        [dictionary setValue:[self fetchDiscount:payment.paymentDiscount] forKey:@"paymentDiscount"];
    }
    if (payment.productIdentifier) {
        [dictionary setValue:payment.productIdentifier forKey:@"productIdentifier"];
    }
    if (payment.requestData) {
        [dictionary setValue:@(payment.requestData.length) forKey:@"requestDataSize"];
    }
    if (payment.applicationUsername) {
        [dictionary setValue:payment.applicationUsername forKey:@"applicationUsername"];
    }
    [dictionary setValue:@(payment.quantity) forKey:@"quantity"];
    [dictionary setValue:@(payment.simulatesAskToBuyInSandbox) forKey:@"simulatesAskToBuyInSandbox"];
    
    return dictionary;
}

- (NSDictionary *)fetchDiscount:(SKPaymentDiscount *)discount {
    NSMutableDictionary *dictionary = [NSMutableDictionary new];
    if (!discount) { return dictionary; }
    
    if (discount.identifier) {
        [dictionary setValue:discount.identifier forKey:@"identifier"];
    }
    if (discount.keyIdentifier) {
        [dictionary setValue:discount.keyIdentifier forKey:@"keyIdentifier"];
    }
    if (discount.nonce) {
        [dictionary setValue:discount.nonce.UUIDString forKey:@"nonceString"];
    }
    if (discount.signature) {
        [dictionary setValue:discount.signature forKey:@"signature"];
    }
    if (discount.timestamp) {
        [dictionary setValue:@(discount.timestamp.doubleValue) forKey:@"timestamp"];
    }
    
    return dictionary;
}

#pragma GCC diagnostic pop

@end
