#pragma once

#include "CoreMinimal.h"
#include "Airbridge/Container/AirbridgeContainer.h"

#if PLATFORM_ANDROID
#include "Android/AndroidJNI.h"
#include "Android/AndroidApplication.h"
#endif

#include "AirbridgeInAppPurchase.generated.h"

/**
 * Represents an Airbridge in-app purchase event.
 */
UCLASS()
class AIRBRIDGEUNREAL_API UAirbridgeInAppPurchase : public UObject
{
	GENERATED_BODY()

public:
#if PLATFORM_ANDROID
	static UAirbridgeInAppPurchase* CreateObject(jobject JavaObject);
#elif PLATFORM_IOS
    static UAirbridgeInAppPurchase* CreateObject(id inAppPurchaseBridge);
#endif

	/**
	 * Sets the semantic attributes for the in-app purchase event.
	 * @param SemanticAttributes The semantic attributes of the in-app purchase event.
	 */
	UFUNCTION(BlueprintCallable, Category = "Airbridge|In-App Purchase")
	void SetSemanticAttributes(const UAirbridgeMap* SemanticAttributes);

	/**
	 * Sets custom attributes for the in-app purchase event.
	 * @param CustomAttributes The custom attributes of the in-app purchase event.
	 */
	UFUNCTION(BlueprintCallable, Category = "Airbridge|In-App Purchase")
	void SetCustomAttributes(const UAirbridgeMap* CustomAttributes);

	/**
	 * Gets the Android-specific purchase data associated with the in-app purchase.
	 * @note This data is only present if the purchase occurred on an <b>Android</b> device.
	 * @return A string containing the purchase's original JSON string
	 *         or an empty string if the platform is not Android or something went wrong during the process.
	 */
	UFUNCTION(BlueprintCallable, Category = "Airbridge|In-App Purchase")
	FString GetPurchase() const;

	/**
	 * Gets the iOS-specific transaction data associated with the in-app purchase.
	 * @note This data is only present if the purchase occurred on an <b>iOS</b> device.
	 * @return A string containing the transaction data
	 *         or an empty string if the platform is not iOS or something went wrong during the process.
	 */
	UFUNCTION(BlueprintCallable, Category = "Airbridge|In-App Purchase")
	FString GetTransaction() const;
    
private:
#if PLATFORM_ANDROID
	JNIEnv* Env = nullptr;
	jobject JavaObject = nullptr;
	
	jmethodID GetMethodID(const char* Name, const char* Sig) const;
#elif PLATFORM_IOS
    id InAppPurchaseBridge;
#endif
};
