// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Kismet/BlueprintFunctionLibrary.h"
#include "AirbridgeConstants.generated.h"

/**
 * Contains category of standard events.
 */
namespace AirbridgeCategory {

    static const FString SIGN_UP                    (TEXT("airbridge.user.signup"));
    static const FString SIGN_IN                    (TEXT("airbridge.user.signin"));
    static const FString SIGN_OUT                   (TEXT("airbridge.user.signout"));

    static const FString HOME_VIEWED                (TEXT("airbridge.ecommerce.home.viewed"));
    static const FString SEARCH_RESULTS_VIEWED      (TEXT("airbridge.ecommerce.searchResults.viewed"));
    static const FString PRODUCT_LIST_VIEWED        (TEXT("airbridge.ecommerce.productList.viewed"));
    static const FString PRODUCT_VIEWED             (TEXT("airbridge.ecommerce.product.viewed"));

    static const FString ADDED_TO_CART              (TEXT("airbridge.ecommerce.product.addedToCart"));
    static const FString ORDER_COMPLETED            (TEXT("airbridge.ecommerce.order.completed"));

    static const FString ADD_PAYMENT_INFO           (TEXT("airbridge.addPaymentInfo"));
    static const FString ADD_TO_WISHLIST            (TEXT("airbridge.addToWishlist"));
    static const FString INITIATE_CHECKOUT          (TEXT("airbridge.initiateCheckout"));
    static const FString ORDER_CANCELED             (TEXT("airbridge.ecommerce.order.canceled"));
    static const FString START_TRIAL                (TEXT("airbridge.startTrial"));
    static const FString SUBSCRIBE                  (TEXT("airbridge.subscribe"));
    static const FString UNSUBSCRIBE                (TEXT("airbridge.unsubscribe"));
    static const FString AD_IMPRESSION              (TEXT("airbridge.adImpression"));
    static const FString AD_CLICK                   (TEXT("airbridge.adClick"));
    static const FString COMPLETE_TUTORIAL          (TEXT("airbridge.completeTutorial"));
    static const FString ACHIEVE_LEVEL              (TEXT("airbridge.achieveLevel"));
    static const FString UNLOCK_ACHIEVEMENT         (TEXT("airbridge.unlockAchievement"));
    static const FString RATE                       (TEXT("airbridge.rate"));
    static const FString SHARE                      (TEXT("airbridge.share"));
    static const FString SCHEDULE                   (TEXT("airbridge.schedule"));
    static const FString SPEND_CREDITS              (TEXT("airbridge.spendCredits"));
}

/**
 * Contains key of semantic attributes.
 */
namespace AirbridgeAttribute {
	
    static const FString ACTION             (TEXT("action"));
    static const FString LABEL              (TEXT("label"));
    static const FString VALUE              (TEXT("value"));
	
    static const FString PRODUCT_LIST_ID    (TEXT("productListID"));
    static const FString CART_ID            (TEXT("cartID"));
    static const FString TRANSACTION_ID     (TEXT("transactionID"));
    static const FString IN_APP_PURCHASED   (TEXT("inAppPurchased"));
    static const FString PRODUCTS           (TEXT("products"));
    static const FString CURRENCY           (TEXT("currency"));
    static const FString TOTAL_VALUE        (TEXT("totalValue"));
    static const FString TOTAL_QUANTITY     (TEXT("totalQuantity"));
	static const FString QUERY              (TEXT("query"));
	
    static const FString PERIOD                                 (TEXT("period"));
    static const FString IS_RENEWAL                             (TEXT("isRenewal"));
    static const FString RENEWAL_COUNT                          (TEXT("renewalCount"));
    static const FString TRANSACTION_TYPE                       (TEXT("transactionType"));
    static const FString TRANSACTION_PAIRED_EVENT_CATEGORY      (TEXT("transactionPairedEventCategory"));
    static const FString TRANSACTION_PAIRED_EVENT_TIMESTAMP     (TEXT("transactionPairedEventTimestamp"));
    static const FString CONTRIBUTION_MARGIN                    (TEXT("contributionMargin"));
    static const FString LIST_ID                                (TEXT("listID"));
    static const FString RATE_ID                                (TEXT("rateID"));
    static const FString RATE                                   (TEXT("rate"));
    static const FString MAX_RATE                               (TEXT("maxRate"));
    static const FString ACHIEVEMENT_ID                         (TEXT("achievementID"));
    static const FString SHARED_CHANNEL                         (TEXT("sharedChannel"));
    static const FString DATE_TIME                              (TEXT("datetime"));
    static const FString DESCRIPTION                            (TEXT("description"));
    static const FString IS_REVENUE                             (TEXT("isRevenue"));
    static const FString PLACE                                  (TEXT("place"));
    static const FString SCHEDULE_ID                            (TEXT("scheduleID"));
    static const FString TYPE                                   (TEXT("type"));
    static const FString LEVEL                                  (TEXT("level"));
    static const FString SCORE                                  (TEXT("score"));
	static const FString AD_PARTNERS							(TEXT("adPartners"));
	static const FString IS_FIRST_PER_USER						(TEXT("isFirstPerUser"));

	static const FString PRODUCT_ID					(TEXT("productID"));
	static const FString PRODUCT_NAME				(TEXT("name"));
	static const FString PRODUCT_PRICE				(TEXT("price"));
	static const FString PRODUCT_QUANTITY			(TEXT("quantity"));
	static const FString PRODUCT_CURRENCY			(TEXT("currency"));
	static const FString PRODUCT_POSITION			(TEXT("position"));
	static const FString PRODUCT_CATEGORY_ID		(TEXT("categoryID"));
	static const FString PRODUCT_CATEGORY_NAME		(TEXT("categoryName"));
	static const FString PRODUCT_BRAND_ID			(TEXT("brandID"));
	static const FString PRODUCT_BRAND_NAME			(TEXT("brandName"));
}

/**
 * Contains key of tracking-link option.
 */
namespace AirbridgeTrackingLinkOption
{
	/// A campaign parameter. (String)
	static const FString CAMPAIGN(TEXT("campaign"));

	/// A campaign parameter. (String)
	static const FString AD_GROUP(TEXT("ad_group"));

	/// A campaign parameter. (String)
	static const FString AD_CREATIVE(TEXT("ad_creative"));

	/// A campaign parameter. (String)
	static const FString CONTENT(TEXT("content"));

	/// A campaign parameter. (String)
	static const FString TERM(TEXT("term"));

	/// A campaign parameter. (String)
	static const FString SUB_ID(TEXT("sub_id"));

	/// A campaign parameter. (String)
	static const FString SUB_ID_1(TEXT("sub_id_1"));

	/// A campaign parameter. (String)
	static const FString SUB_ID_2(TEXT("sub_id_2"));

	/// A campaign parameter. (String)
	static const FString SUB_ID_3(TEXT("sub_id_3"));

	/// A url of deeplink open the app. (Custom Scheme URL String)
	static const FString DEEPLINK_URL(TEXT("deeplink_url"));

	/// A option enable stop-over-feature of deeplink. (Boolean)
	static const FString DEEPLINK_STOPOVER(TEXT("deeplink_stopover"));

	/// A url of fallback for ios if app is not installed. ("store" | Web URL String)
	static const FString FALLBACK_IOS(TEXT("fallback_ios"));

	/// A url of fallback for android if app is not installed. ("store" | Web URL String)
	static const FString FALLBACK_ANDROID(TEXT("fallback_android"));

	/// A url of fallback for desktop if app is not installed. (Web URL String)
	static const FString FALLBACK_DESKTOP(TEXT("fallback_desktop"));

	/// A custom id of store for ios used when user move to store. (String)
	static const FString FALLBACK_IOS_STORE_PPID(TEXT("fallback_ios_store_ppid"));

	/// A custom id of store for android used when user move to store. (String)
	static const FString FALLBACK_ANDROID_STORE_LISTING(TEXT("fallback_android_store_listing"));

	/// A title of link-preview. (String)
	static const FString OGTAG_TITLE(TEXT("ogtag_title"));

	/// A description of link-preview. (String)
	static const FString OGTAG_DESCRIPTION(TEXT("ogtag_description"));

	/// A image-url of link-preview. (Web URL String)
	static const FString OGTAG_IMAGE_URL(TEXT("ogtag_image_url"));

	/// A customization of link-preview. ("desktop")
	static const FString OGTAG_WEBSITE_CRAWL(TEXT("ogtag_website_crawl"));

	/// A custom-short-id of tracking-link. (String)
	static const FString CUSTOM_SHORT_ID(TEXT("custom_short_id"));

	/// A option enable re-engagement-feature of attribution. ("off" | "on_true" | "on_false")
	static const FString IS_REENGAGEMENT(TEXT("is_reengagement"));
}

/** @cond HIDDEN_SYMBOLS */
UCLASS()
class AIRBRIDGEUNREAL_API UAirbridgeConstants : public UBlueprintFunctionLibrary
{
	GENERATED_BODY()

#pragma region AirbridgeCategory
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SIGN_UP")
	static UPARAM(DisplayName = "Category") FString GetCategorySignUp()
	{
		return AirbridgeCategory::SIGN_UP;
	}
 
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SIGN_IN")
	static UPARAM(DisplayName = "Category") FString GetCategorySignIn()
	{       
		return AirbridgeCategory::SIGN_IN;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SIGN_OUT")
	static UPARAM(DisplayName = "Category") FString GetCategorySignOut()
	{       
		return AirbridgeCategory::SIGN_OUT;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::HOME_VIEWED")
	static UPARAM(DisplayName = "Category") FString GetCategoryHomeViewed()
	{       
		return AirbridgeCategory::HOME_VIEWED;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SEARCH_RESULTS_VIEWED")
	static UPARAM(DisplayName = "Category") FString GetCategorySearchResultsViewed()
	{       
		return AirbridgeCategory::SEARCH_RESULTS_VIEWED;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::PRODUCT_LIST_VIEWED")
	static UPARAM(DisplayName = "Category") FString GetCategoryProductListViewed()
	{       
		return AirbridgeCategory::PRODUCT_LIST_VIEWED;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::PRODUCT_VIEWED")
	static UPARAM(DisplayName = "Category") FString GetCategoryProductViewed()
	{       
		return AirbridgeCategory::PRODUCT_VIEWED;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::ADDED_TO_CART")
	static UPARAM(DisplayName = "Category") FString GetCategoryAddedToCart()
	{       
		return AirbridgeCategory::ADDED_TO_CART;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::ORDER_COMPLETED")
	static UPARAM(DisplayName = "Category") FString GetCategoryOrderCompleted()
	{       
		return AirbridgeCategory::ORDER_COMPLETED;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::ADD_PAYMENT_INFO")
	static UPARAM(DisplayName = "Category") FString GetCategoryAddPaymentInfo()
	{       
		return AirbridgeCategory::ADD_PAYMENT_INFO;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::ADD_TO_WISHLIST")
	static UPARAM(DisplayName = "Category") FString GetCategoryAddToWishlist()
	{       
		return AirbridgeCategory::ADD_TO_WISHLIST;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::INITIATE_CHECKOUT")
	static UPARAM(DisplayName = "Category") FString GetCategoryInitiateCheckout()
	{       
		return AirbridgeCategory::INITIATE_CHECKOUT;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::ORDER_CANCELED")
	static UPARAM(DisplayName = "Category") FString GetCategoryOrderCanceled()
	{       
		return AirbridgeCategory::ORDER_CANCELED;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::START_TRIAL")
	static UPARAM(DisplayName = "Category") FString GetCategoryStartTrial()
	{       
		return AirbridgeCategory::START_TRIAL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SUBSCRIBE")
	static UPARAM(DisplayName = "Category") FString GetCategorySubscribe()
	{       
		return AirbridgeCategory::SUBSCRIBE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::UNSUBSCRIBE")
	static UPARAM(DisplayName = "Category") FString GetCategoryUnsubscribe()
	{       
		return AirbridgeCategory::UNSUBSCRIBE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::AD_IMPRESSION")
	static UPARAM(DisplayName = "Category") FString GetCategoryAdImpression()
	{       
		return AirbridgeCategory::AD_IMPRESSION;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::AD_CLICK")
	static UPARAM(DisplayName = "Category") FString GetCategoryAdClick()
	{       
		return AirbridgeCategory::AD_CLICK;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::COMPLETE_TUTORIAL")
	static UPARAM(DisplayName = "Category") FString GetCategoryCompleteTutorial()
	{       
		return AirbridgeCategory::COMPLETE_TUTORIAL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::ACHIEVE_LEVEL")
	static UPARAM(DisplayName = "Category") FString GetCategoryAchieveLevel()
	{       
		return AirbridgeCategory::ACHIEVE_LEVEL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::UNLOCK_ACHIEVEMENT")
	static UPARAM(DisplayName = "Category") FString GetCategoryUnlockAchievement()
	{       
		return AirbridgeCategory::UNLOCK_ACHIEVEMENT;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::RATE")
	static UPARAM(DisplayName = "Category") FString GetCategoryRate()
	{       
		return AirbridgeCategory::RATE;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SHARE")
	static UPARAM(DisplayName = "Category") FString GetCategoryShare()
	{       
		return AirbridgeCategory::SHARE;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SCHEDULE")
	static UPARAM(DisplayName = "Category") FString GetCategorySchedule()
	{       
		return AirbridgeCategory::SCHEDULE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Category", DisplayName = "AirbridgeCategory::SPEND_CREDITS")
	static UPARAM(DisplayName = "Category") FString GetCategorySpendCredits()
	{       
		return AirbridgeCategory::SPEND_CREDITS;
	}

#pragma endregion

#pragma region AirbridgeAttribute

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::ACTION")
	static UPARAM(DisplayName = "Key") FString GetAttributeAction()
	{
		return AirbridgeAttribute::ACTION;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::LABEL")
	static UPARAM(DisplayName = "Key") FString GetAttributeLabel()
	{
		return AirbridgeAttribute::LABEL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::VALUE")
	static UPARAM(DisplayName = "Key") FString GetAttributeValue()
	{
		return AirbridgeAttribute::VALUE;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_LIST_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductListID()
	{
		return AirbridgeAttribute::PRODUCT_LIST_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::CART_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeCartID()
	{
		return AirbridgeAttribute::CART_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TRANSACTION_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeTransactionID()
	{
		return AirbridgeAttribute::TRANSACTION_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::IN_APP_PURCHASED")
	static UPARAM(DisplayName = "Key") FString GetAttributeInAppPurchased()
	{
		return AirbridgeAttribute::IN_APP_PURCHASED;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCTS")
	static UPARAM(DisplayName = "Key") FString GetAttributeProducts()
	{
		return AirbridgeAttribute::PRODUCTS;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::CURRENCY")
	static UPARAM(DisplayName = "Key") FString GetAttributeCurrency()
	{
		return AirbridgeAttribute::CURRENCY;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TOTAL_VALUE")
	static UPARAM(DisplayName = "Key") FString GetAttributeTotalValue()
	{
		return AirbridgeAttribute::TOTAL_VALUE;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TOTAL_QUANTITY")
	static UPARAM(DisplayName = "Key") FString GetAttributeTotalQuantity()
	{
		return AirbridgeAttribute::TOTAL_QUANTITY;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::QUERY")
	static UPARAM(DisplayName = "Key") FString GetAttributeQuery()
	{
		return AirbridgeAttribute::QUERY;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PERIOD")
	static UPARAM(DisplayName = "Key") FString GetAttributePeriod()
	{
		return AirbridgeAttribute::PERIOD;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::IS_RENEWAL")
	static UPARAM(DisplayName = "Key") FString GetAttributeIsRenewal()
	{
		return AirbridgeAttribute::IS_RENEWAL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::RENEWAL_COUNT")
	static UPARAM(DisplayName = "Key") FString GetAttributeRenewalCount()
	{
		return AirbridgeAttribute::RENEWAL_COUNT;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TRANSACTION_TYPE")
	static UPARAM(DisplayName = "Key") FString GetAttributeTransactionType()
	{
		return AirbridgeAttribute::TRANSACTION_TYPE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TRANSACTION_PAIRED_EVENT_CATEGORY")
	static UPARAM(DisplayName = "Key") FString GetAttributeTransactionPairedEventCategory()
	{
		return AirbridgeAttribute::TRANSACTION_PAIRED_EVENT_CATEGORY;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TRANSACTION_PAIRED_EVENT_TIMESTAMP")
	static UPARAM(DisplayName = "Key") FString GetAttributeTransactionPairedEventTimestamp()
	{
		return AirbridgeAttribute::TRANSACTION_PAIRED_EVENT_TIMESTAMP;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::CONTRIBUTION_MARGIN")
	static UPARAM(DisplayName = "Key") FString GetAttributeContributionMargin()
	{
		return AirbridgeAttribute::CONTRIBUTION_MARGIN;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::LIST_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeListID()
	{
		return AirbridgeAttribute::LIST_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::RATE_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeRateID()
	{
		return AirbridgeAttribute::RATE_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::RATE")
	static UPARAM(DisplayName = "Key") FString GetAttributeRate()
	{
		return AirbridgeAttribute::RATE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::MAX_RATE")
	static UPARAM(DisplayName = "Key") FString GetAttributeMaxRate()
	{
		return AirbridgeAttribute::MAX_RATE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::ACHIEVEMENT_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeAchievementID()
	{
		return AirbridgeAttribute::ACHIEVEMENT_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::SHARED_CHANNEL")
	static UPARAM(DisplayName = "Key") FString GetAttributeSharedChannel()
	{
		return AirbridgeAttribute::SHARED_CHANNEL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::DATE_TIME")
	static UPARAM(DisplayName = "Key") FString GetAttributeDatetime()
	{
		return AirbridgeAttribute::DATE_TIME;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::DESCRIPTION")
	static UPARAM(DisplayName = "Key") FString GetAttributeDescription()
	{
		return AirbridgeAttribute::DESCRIPTION;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::IS_REVENUE")
	static UPARAM(DisplayName = "Key") FString GetAttributeIsRevenue()
	{
		return AirbridgeAttribute::IS_REVENUE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PLACE")
	static UPARAM(DisplayName = "Key") FString GetAttributePlace()
	{
		return AirbridgeAttribute::PLACE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::SCHEDULE_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeScheduleID()
	{
		return AirbridgeAttribute::SCHEDULE_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::TYPE")
	static UPARAM(DisplayName = "Key") FString GetAttributeType()
	{
		return AirbridgeAttribute::TYPE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::LEVEL")
	static UPARAM(DisplayName = "Key") FString GetAttributeLevel()
	{
		return AirbridgeAttribute::LEVEL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::SCORE")
	static UPARAM(DisplayName = "Key") FString GetAttributeScore()
	{
		return AirbridgeAttribute::SCORE;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::AD_PARTNERS")
	static UPARAM(DisplayName = "Key") FString GetAttributeAdPartners()
	{
		return AirbridgeAttribute::AD_PARTNERS;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::IS_FIRST_PER_USER")
	static UPARAM(DisplayName = "Key") FString GetAttributeIsFirstPerUser()
	{
		return AirbridgeAttribute::IS_FIRST_PER_USER;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductID()
	{
		return AirbridgeAttribute::PRODUCT_ID;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_NAME")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductName()
	{
		return AirbridgeAttribute::PRODUCT_NAME;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_PRICE")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductPrice()
	{
		return AirbridgeAttribute::PRODUCT_PRICE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_QUANTITY")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductQuantity()
	{
		return AirbridgeAttribute::PRODUCT_QUANTITY;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_CURRENCY")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductCurrency()
	{
		return AirbridgeAttribute::PRODUCT_CURRENCY;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_POSITION")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductPosition()
	{
		return AirbridgeAttribute::PRODUCT_POSITION;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_CATEGORY_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductCategoryID()
	{
		return AirbridgeAttribute::PRODUCT_CATEGORY_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_CATEGORY_NAME")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductCategoryName()
	{
		return AirbridgeAttribute::PRODUCT_CATEGORY_NAME;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_BRAND_ID")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductBrandID()
	{
		return AirbridgeAttribute::PRODUCT_BRAND_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Attribute", DisplayName = "AirbridgeAttribute::PRODUCT_BRAND_NAME")
	static UPARAM(DisplayName = "Key") FString GetAttributeProductBrandName()
	{
		return AirbridgeAttribute::PRODUCT_BRAND_NAME;
	}

#pragma endregion

#pragma region AirbridgeTrackingLinkOption

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::CAMPAIGN")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetCampaign()
	{
		return AirbridgeTrackingLinkOption::CAMPAIGN;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::AD_GROUP")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetAdGroup()
	{
		return AirbridgeTrackingLinkOption::AD_GROUP;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::AD_CREATIVE")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetAdCreative()
	{
		return AirbridgeTrackingLinkOption::AD_CREATIVE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::CONTENT")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetContent()
	{
		return AirbridgeTrackingLinkOption::CONTENT;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::TERM")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetTerm()
	{
		return AirbridgeTrackingLinkOption::TERM;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::SUB_ID")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetSubId()
	{
		return AirbridgeTrackingLinkOption::SUB_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::SUB_ID_1")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetSubId1()
	{
		return AirbridgeTrackingLinkOption::SUB_ID_1;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::SUB_ID_2")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetSubId2()
	{
		return AirbridgeTrackingLinkOption::SUB_ID_2;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::SUB_ID_3")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetSubId3()
	{
		return AirbridgeTrackingLinkOption::SUB_ID_3;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::DEEPLINK_URL")
	static UPARAM(DisplayName = "Tracking Link Option") FString Get()
	{
		return AirbridgeTrackingLinkOption::DEEPLINK_URL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::DEEPLINK_STOPOVER")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetDeeplinkStopover()
	{
		return AirbridgeTrackingLinkOption::DEEPLINK_STOPOVER;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::FALLBACK_IOS")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetFallbackIos()
	{
		return AirbridgeTrackingLinkOption::FALLBACK_IOS;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::FALLBACK_ANDROID")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetFallbackAndroid()
	{
		return AirbridgeTrackingLinkOption::FALLBACK_ANDROID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::FALLBACK_DESKTOP")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetFallbackDesktop()
	{
		return AirbridgeTrackingLinkOption::FALLBACK_DESKTOP;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::FALLBACK_IOS_STORE_PPID")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetFallbackIosStorePpid()
	{
		return AirbridgeTrackingLinkOption::FALLBACK_IOS_STORE_PPID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::FALLBACK_ANDROID_STORE_LISTING")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetFallbackAndroidStoreListing()
	{
		return AirbridgeTrackingLinkOption::FALLBACK_ANDROID_STORE_LISTING;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::OGTAG_TITLE")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetOgtagTitle()
	{
		return AirbridgeTrackingLinkOption::OGTAG_TITLE;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::OGTAG_DESCRIPTION")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetOgtagDescription()
	{
		return AirbridgeTrackingLinkOption::OGTAG_DESCRIPTION;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::OGTAG_IMAGE_URL")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetOgtagImageUrl()
	{
		return AirbridgeTrackingLinkOption::OGTAG_IMAGE_URL;
	}
	
	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::OGTAG_WEBSITE_CRAWL")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetOgtagWebsiteCrawl()
	{
		return AirbridgeTrackingLinkOption::OGTAG_WEBSITE_CRAWL;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::CUSTOM_SHORT_ID")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetCustomShortId()
	{
		return AirbridgeTrackingLinkOption::CUSTOM_SHORT_ID;
	}

	UFUNCTION(BlueprintPure, Category = "Airbridge Constants|Tracking Link Option", DisplayName = "AirbridgeTrackingLinkOption::IS_REENGAGEMENT")
	static UPARAM(DisplayName = "Tracking Link Option") FString GetIsReengagement()
	{
		return AirbridgeTrackingLinkOption::IS_REENGAGEMENT;
	}

#pragma endregion
};
/** @endcond */